#
# (C) Tenable Network Security
#
# Modified by Nicolas Pouvesle to actually test for the bug
# actual test will *not* DoS dropbear
if (description) {
  script_id(14234);
  script_bugtraq_id(10803);
  script_version ("$Revision: 1.4 $");

  if (defined_func("script_xref")) {
    script_xref(name:"OSVDB", value:"8137");
  }

  name["english"] = "Dropbear remote DSS SSH vuln";
  script_name(english:name["english"]);
 
  desc["english"] = "
The remote host is running Dropbear prior to version 0.43.  
There is a flaw in this version of Dropbear which would
enable a remote attacker to gain control of the system
from a remote location.

Solution : Upgrade to at least version 0.43 of Dropbear. 

See also : http://www.securityfocus.com/bid/10803 
 
Risk Factor : High";
  script_description(english:desc["english"]);
 
  summary["english"] = "Dropbear remote DSS SSH vuln check";
  script_summary(english:summary["english"]);
 
  script_category(ACT_GATHER_INFO);
  script_copyright(english:"This script is Copyright (C) Tenable Network Security");
  family["english"] = "Gain root remotely";
  script_family(english:family["english"]);

  script_require_ports("Services/ssh", 22);
  script_dependencie("ssh_detect.nasl");
  exit(0);
}


include ('ssh_func.inc');

port = get_kb_item ("Services/ssh");
if (!port)
  port = 22;
if (!get_port_state (port))
  exit (0);

login = kb_ssh_login ();
pub = kb_ssh_publickey ();
priv = kb_ssh_privatekey ();
pass = kb_ssh_passphrase ();

if (login && pub && priv)
{
 soc = open_sock_tcp (port);
 ret = ssh_login (socket: soc, login: login, password: NULL, pub: pub, priv: priv, passphrase:pass);

 close (soc);

 if (ret == 0)
 {
	authenticated = 0;
  	socket = open_sock_tcp (port);
  	if (socket)
  	{
   		server_user = tolower (login);
   		init ();
   		server_version = ssh_exchange_identification (socket:socket);
   
   		# Like ssh_login has succeeded we don't need to test return
   		# It means SSH server is not vulnerable.
   		# However I prefer let if (!xxx) {close();exit()} ...

   		if (!server_version)
   		{
     			close (socket);
     			exit(0);
   		}
   
   		ret = ssh_kex2 (socket: socket, server_version:server_version);
   		if (ret!= 0)
   		{
     			close (socket);
     			exit(0);
   		}

   		payload = putstring (buffer:"ssh-userauth");
   		send_ssh_packet (socket: socket, payload: payload, code:raw_string (0x05));
   		payload = recv_ssh_packet (socket:socket);
   		if (ord (payload[0]) != 6)
   		{
     			close (socket);
     			exit(0);
   		}

   		payload = putstring (buffer: server_user) + putstring (buffer: "ssh-connection") + putstring (buffer:"none");
   		send_ssh_packet (socket: socket, payload: payload, code: raw_int8 (i:50));

   		payload = recv_ssh_packet (socket:socket);
   		if (ord (payload[0]) != 51)
   		{
     			close (socket);
     			exit(0);
   		}

   		support = getstring (buffer: payload, pos:1);
   		if (!ereg (string: support, pattern:"publickey"))
   		{
     			close (socket);
     			exit(0);
   		}

   		blobpub = base64decode (str:pub);
   		payload = putstring (buffer: server_user) + putstring (buffer: "ssh-connection") + putstring (buffer: "publickey") + raw_int8 (i:1);
   		to_hash = putstring (buffer: session_id) + raw_int8 (i:50);
   		typestr = getstring (buffer: blobpub, pos:0);

   		### We must not quit here because user provide a RSA key !!!
   		if (typestr >< "ssh-dss")
   		{
    			next = 4 + strlen (typestr);
    			p = getstring (buffer: blobpub, pos:next);
    			next = next + 4 + strlen (p);
    			q = getstring (buffer: blobpub, pos:next);
    			next = next + 4 + strlen (q);
    			g = getstring (buffer: blobpub, pos:next);
    			next = next + 4 + strlen (g);
    			pub_key = getstring (buffer: blobpub, pos:next);
    			privkey = pem_to_dsa (priv: priv, passphrase:passphrase);
    			if (!privkey)
    			{
      				close (socket);
      				exit(0);
    			}

    			payload += putstring (buffer: "ssh-dss") + putstring (buffer:blobpub);
    			to_hash += payload;
    			hash = SHA1 (to_hash);
    			signature = dsa_do_sign (p: p, q: q, g: g, pub: pub_key, priv: privkey, data:hash);
 
    			if (!signature)
    			{
      				close (socket);
      				exit(0);
    			}
           
    			signature = putstring (buffer: "ssh-dss") + crap (data: raw_string (0x00), length:44);
    			payload += putstring (buffer:signature);
    			send_ssh_packet (socket: socket, payload: payload, code: raw_int8 (i:50));
    			payload = recv_ssh_packet (socket:socket);

    			if (strlen (payload) > 1)
      				authenticated = 1;
            
    			close (socket);
           
    			if (!authenticated)
     				security_hole (port);
    			exit(0);
   		}
  	}
 }
}

close (socket);

##### If we can not test flaw, we can test banner ######

#[root@f00dikator dropbear-0.43]# telnet localhost 22
#Trying 127.0.0.1...
#Connected to localhost.
#Escape character is '^]'.
#SSH-2.0-dropbear_0.43

banner = get_kb_item("SSH/banner/" + port );

if (! banner)
  exit(0);

  desc = "
The remote host is running Dropbear prior to version 0.43.  
There is a flaw in this version of Dropbear which would
enable a remote attacker to gain control of the system
from a remote location.

*** Nessus reports this vulnerability using only
*** information that was gathered. To really test
*** the vulnerability Nessus need an SSH account with
*** correct dsa keys.

Solution : Upgrade to at least version 0.43 of Dropbear. 

See also : http://www.securityfocus.com/bid/10803 
 
Risk Factor : High";

# version 0.28 thru 0.42 are vulnerable
if (egrep(string:banner, pattern:"-dropbear_0\.(2[89]|3[0-9]|4[0-2])") )
	security_hole(port:port, data:desc);

